from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class QuoraNLRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="Quora-NL",
        dataset={
            "path": "clips/beir-nl-quora",
            "revision": "698f8a5922a49e50e3c71b52a2e9d63b22756843",
        },
        description=(
            "QuoraRetrieval is based on questions that are marked as duplicates on the Quora platform. Given a"
            + " question, find other (duplicate) questions. QuoraRetrieval-NL is a Dutch translation."
        ),
        reference="https://huggingface.co/datasets/clips/beir-nl-quora",
        type="Retrieval",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="ndcg_at_10",
        date=("2016-03-01", "2016-03-01"),
        domains=["Written"],
        task_subtypes=["Duplicate Detection"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated and verified",  # manually checked a small subset
        bibtex_citation="""@misc{banar2024beirnlzeroshotinformationretrieval,
    title={BEIR-NL: Zero-shot Information Retrieval Benchmark for the Dutch Language},
     author={Nikolay Banar and Ehsan Lotfi and Walter Daelemans},
     year={2024},
     eprint={2412.08329},
     archivePrefix={arXiv},
     primaryClass={cs.CL},
     url={https://arxiv.org/abs/2412.08329},
}""",
        adapted_from=["QuoraRetrieval"],
    )



from __future__ import annotations

import logging
import os
import time
from functools import partial
from typing import Any

import numpy as np
import tqdm

from mteb.encoder_interface import PromptType
from mteb.model_meta import ModelMeta
from mteb.models.wrapper import Wrapper
from mteb.requires_package import requires_package

logger = logging.getLogger(__name__)


class SeedTextEmbeddingModel(Wrapper):
    def __init__(
        self,
        model_name: str,
        rate_limit_per_minute: int = 300,
        **kwargs,
    ) -> None:
        requires_package(
            self,
            "openai",
            model_name,
            install_instruction="pip install 'mteb[openai]'",
        )
        from openai import OpenAI

        requires_package(
            self,
            "tiktoken",
            model_name,
            install_instruction="pip install 'mteb[openai]'",
        )
        import tiktoken

        self.model_name = model_name
        self.rate_limit_per_minute = rate_limit_per_minute
        self.last_request_time = 0
        self.tokenizer = tiktoken.get_encoding("cl100k_base")
        self.client = OpenAI(
            api_key=os.environ["ARK_API_KEY"],
            base_url="https://ark.cn-beijing.volces.com/api/v3",
        )

    def _enforce_rate_limit(self):
        """Enforce rate limiting"""
        current_time = time.time()
        time_since_last_request = current_time - self.last_request_time
        min_interval = 60.0 / self.rate_limit_per_minute

        if time_since_last_request < min_interval:
            time.sleep(min_interval - time_since_last_request)

        self.last_request_time = time.time()

    def _truncate_text(self, text: str, max_tokens: int = 32000) -> str:
        """Truncate text to fit within token limit"""
        tokens = self.tokenizer.encode(text)
        if len(tokens) > max_tokens:
            tokens = tokens[:max_tokens]
            text = self.tokenizer.decode(tokens)
        return text

    def _format_instruction(self, instruction: str, input_: str) -> str:
        if isinstance(instruction, dict):
            return input_
        elif isinstance(instruction, str) and len(instruction):
            return instruction + "\n" + input_
        else:
            return input_

    def _embed(
        self,
        sentences: list[str],
        instruction: str,
        show_progress_bar: bool = False,
        retries: int = 5,
    ) -> np.ndarray:
        max_batch_size = 20
        batches = [
            sentences[i : i + max_batch_size]
            for i in range(0, len(sentences), max_batch_size)
        ]

        all_embeddings = []

        for batch in tqdm.tqdm(batches, leave=False, disable=not show_progress_bar):
            # Truncate texts
            batch = [self._truncate_text(text) for text in batch]

            # Add instruction to each text
            batch = [self._format_instruction(instruction, text) for text in batch]

            while retries > 0:
                try:
                    self._enforce_rate_limit()
                    response = self.client.embeddings.create(
                        model=self.model_name, input=batch, encoding_format="float"
                    )
                    embeddings = [x.embedding for x in response.data]
                    break
                except Exception as e:
                    logger.warning(
                        f"Retrying... {retries} retries left. Error: {str(e)}"
                    )
                    retries -= 1
                    if retries == 0:
                        raise e

            all_embeddings.extend(embeddings)

        return np.array(all_embeddings)

    def encode(
        self,
        sentences: list[str],
        *,
        task_name: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> np.ndarray:
        logger.warning("The API will be publicly available soon. Stay tuned!")

        instruction = self.get_instruction(task_name, prompt_type)
        show_progress_bar = kwargs.pop("show_progress_bar", False)

        return self._embed(
            sentences,
            instruction=instruction,
            show_progress_bar=show_progress_bar,
        )


seed_embedding = ModelMeta(
    name="ByteDance-Seed/Doubao-1.5-Embedding",
    revision="2",
    release_date="2025-04-25",
    languages=[
        "eng-Latn",
        "zho-Hans",
    ],
    loader=partial(
        SeedTextEmbeddingModel,
        model_name="doubao-1-5-embedding",
        rate_limit_per_minute=300,
    ),
    max_tokens=32768,
    embed_dim=2048,
    open_weights=False,
    n_parameters=None,
    memory_usage_mb=None,
    license=None,
    reference="https://huggingface.co/ByteDance-Seed/Doubao-1.5-Embedding",
    similarity_fn_name="cosine",
    framework=["API"],
    use_instructions=True,
    training_datasets=None,
    public_training_code=None,
    public_training_data=None,
)



from __future__ import annotations

from importlib.metadata import version

from mteb.benchmarks.benchmarks import (
    MTEB_ENG_CLASSIC,
    MTEB_MAIN_RU,
    MTEB_RETRIEVAL_LAW,
    MTEB_RETRIEVAL_MEDICAL,
    MTEB_RETRIEVAL_WITH_INSTRUCTIONS,
    CoIR,
)
from mteb.evaluation import *
from mteb.load_results import BenchmarkResults, load_results
from mteb.models import get_model, get_model_meta, get_model_metas
from mteb.overview import TASKS_REGISTRY, get_task, get_tasks

from .benchmarks.benchmarks import Benchmark
from .benchmarks.get_benchmark import BENCHMARK_REGISTRY, get_benchmark, get_benchmarks

__version__ = version("mteb")  # fetch version from install metadata


__all__ = [
    "MTEB_ENG_CLASSIC",
    "MTEB_MAIN_RU",
    "MTEB_RETRIEVAL_LAW",
    "MTEB_RETRIEVAL_MEDICAL",
    "MTEB_RETRIEVAL_WITH_INSTRUCTIONS",
    "CoIR",
    "TASKS_REGISTRY",
    "get_tasks",
    "get_task",
    "get_model",
    "get_model_meta",
    "get_model_metas",
    "load_results",
    "Benchmark",
    "get_benchmark",
    "get_benchmarks",
    "BenchmarkResults",
    "BENCHMARK_REGISTRY",
]



from __future__ import annotations

from datasets import Dataset, DatasetDict, load_dataset
from huggingface_hub import create_repo

ds = load_dataset("timpal0l/scandisent")
repo_name = "mteb/scandisent"
create_repo(repo_name, repo_type="dataset")

ds1 = {}
df_split = ds["train"].to_polars()
df_grouped = dict(df_split.group_by(["language"]))
for lang in set(df_split["language"].unique()):
    ds1.setdefault(lang, {})
    # Remove lang column and convert back to HF datasets, not strictly necessary but better for compatibility
    ds1[lang] = DatasetDict(
        {
            "train": Dataset.from_polars(df_grouped[(lang,)].drop("language")).select(
                range(7500)
            ),
            "test": Dataset.from_polars(df_grouped[(lang,)].drop("language")).select(
                range(7500, 10000)
            ),
        }
    )
    ds1[lang].push_to_hub(repo_name, config_name=lang)
